@@ -17,9 +17,10 @@ git checkout -b "$target" "$release"
 
 # Update openshift's master and take all needed files from there.
 git fetch openshift master
-git checkout openshift/master -- openshift OWNERS_ALIASES OWNERS Makefile
+git checkout openshift/master -- openshift OWNERS_ALIASES OWNERS Makefile content_sets.yml container.yaml
 make generate-dockerfiles
+make generate-p12n-dockerfiles
 make RELEASE=$release generate-release
 make RELEASE=ci generate-release
-git add openshift OWNERS_ALIASES OWNERS Makefile
+git add openshift OWNERS_ALIASES OWNERS Makefile content_sets.yml container.yaml
 git commit -m "Add openshift specific files."
