@@ -21,8 +21,8 @@ import (
 
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
-	"github.com/knative/pkg/kmeta"
-	"github.com/knative/serving/pkg/apis/serving/v1alpha1"
+	"knative.dev/pkg/kmeta"
+	"knative.dev/serving/pkg/apis/serving/v1alpha1"
 )
 
 func TestNamer(t *testing.T) {
