@@ -25,10 +25,12 @@ E2E_CLUSTER_MACHINE=${E2E_CLUSTER_MACHINE:-n1-standard-8}
 # This script provides helper methods to perform cluster actions.
 source $(dirname $0)/../vendor/knative.dev/test-infra/scripts/e2e-tests.sh
 
-# Default Istio configuration to install: 1.2-latest, no mesh, cert manager 0.6.1.
-ISTIO_VERSION="1.2-latest"
-ISTIO_MESH=0
-CERT_MANAGER_VERSION="0.6.1"
+CERT_MANAGER_VERSION="0.9.1"
+ISTIO_VERSION=""
+GLOO_VERSION=""
+KOURIER_VERSION=""
+
+HTTPS=0
 
 # Current YAMLs used to install Knative Serving.
 INSTALL_RELEASE_YAML=""
@@ -36,7 +38,8 @@ INSTALL_MONITORING_YAML=""
 
 INSTALL_MONITORING=0
 
-INSTALL_BETA=1
+GATEWAY_SETUP=0
+RECONCILE_GATEWAY=0
 
 # List of custom YAMLs to install, if specified (space-separated).
 INSTALL_CUSTOM_YAMLS=""
@@ -47,6 +50,7 @@ function parse_flags() {
     --istio-version)
       [[ $2 =~ ^[0-9]+\.[0-9]+(\.[0-9]+|\-latest)$ ]] || abort "version format must be '[0-9].[0-9].[0-9]' or '[0-9].[0-9]-latest"
       readonly ISTIO_VERSION=$2
+      GATEWAY_SETUP=1
       return 2
       ;;
     --version)
@@ -60,23 +64,23 @@ function parse_flags() {
       return 2
       ;;
     --mesh)
-      readonly ISTIO_MESH=1
+      readonly MESH=1
       return 1
       ;;
     --no-mesh)
-      readonly ISTIO_MESH=0
+      readonly MESH=0
+      return 1
+      ;;
+    --https)
+      readonly HTTPS=1
       return 1
       ;;
     --install-monitoring)
       readonly INSTALL_MONITORING=1
       return 1
       ;;
-    --install-alpha)
-      readonly INSTALL_BETA=0
-      return 1
-      ;;
-    --install-beta)
-      readonly INSTALL_BETA=1
+    --reconcile-gateway)
+      readonly RECONCILE_GATEWAY=1
       return 1
       ;;
     --custom-yamls)
@@ -87,8 +91,17 @@ function parse_flags() {
       return 2
       ;;
     --gloo-version)
-      [[ $2 =~ ^[0-9]+\.[0-9]+(\.[0-9]+|\-latest)$ ]] || abort "version format must be '[0-9].[0-9].[0-9]' or '[0-9].[0-9]-latest"
+      # currently, the value of --gloo-version is ignored
+      # latest version of Gloo pinned in third_party will be installed
       readonly GLOO_VERSION=$2
+      GATEWAY_SETUP=1
+      return 2
+      ;;
+    --kourier-version)
+      # currently, the value of --kourier-version is ignored
+      # latest version of Kourier pinned in third_party will be installed
+      readonly KOURIER_VERSION=$2
+      GATEWAY_SETUP=1
       return 2
       ;;
   esac
@@ -102,8 +115,8 @@ function parse_flags() {
 function build_knative_from_source() {
   local YAML_LIST="$(mktemp)"
 
-  # set ko flags to omit istio resources from generated YAMLs
-  if [[ -n "${GLOO_VERSION}" ]]; then
+  # Set ko flags to omit istio resources from generated YAMLs
+  if [[ -z "${ISTIO_VERSION}" ]]; then
     KO_FLAGS="${KO_FLAGS} --selector=networking.knative.dev/ingress-provider!=istio"
   fi
 
@@ -131,7 +144,6 @@ function install_knative_serving() {
     return
   fi
   echo ">> Installing Knative serving from custom YAMLs"
-  kubectl label namespace default istio-injection=enabled
   echo "Custom YAML files: ${INSTALL_CUSTOM_YAMLS}"
   for yaml in ${INSTALL_CUSTOM_YAMLS}; do
     echo "Installing '${yaml}'"
@@ -142,7 +154,7 @@ function install_knative_serving() {
 function install_istio() {
   local istio_base="./third_party/istio-${ISTIO_VERSION}"
   INSTALL_ISTIO_CRD_YAML="${istio_base}/istio-crds.yaml"
-  (( ISTIO_MESH )) && INSTALL_ISTIO_YAML="${istio_base}/istio.yaml" || INSTALL_ISTIO_YAML="${istio_base}/istio-lean.yaml"
+  (( MESH )) && INSTALL_ISTIO_YAML="${istio_base}/istio.yaml" || INSTALL_ISTIO_YAML="${istio_base}/istio-lean.yaml"
 
   echo "Istio CRD YAML: ${INSTALL_ISTIO_CRD_YAML}"
   echo "Istio YAML: ${INSTALL_ISTIO_YAML}"
@@ -162,7 +174,7 @@ function install_istio() {
 }
 
 function install_gloo() {
-  local gloo_base="./third_party/gloo-${GLOO_VERSION}"
+  local gloo_base="./third_party/gloo-latest"
   INSTALL_GLOO_YAML="${gloo_base}/gloo.yaml"
   echo "Gloo YAML: ${INSTALL_GLOO_YAML}"
   echo ">> Bringing up Gloo"
@@ -170,6 +182,15 @@ function install_gloo() {
   kubectl apply -f ${INSTALL_GLOO_YAML} || return 1
 }
 
+function install_kourier() {
+  local kourier_base="./third_party/kourier-latest"
+  INSTALL_KOURIER_YAML="${kourier_base}/kourier.yaml"
+  echo "Kourier YAML: ${INSTALL_KOURIER_YAML}"
+  echo ">> Bringing up Kourier"
+
+  kubectl apply -f ${INSTALL_KOURIER_YAML} || return 1
+}
+
 # Installs Knative Serving in the current cluster, and waits for it to be ready.
 # If no parameters are passed, installs the current source-based build.
 # Parameters: $1 - Knative Serving YAML file
@@ -180,11 +201,13 @@ function install_knative_serving_standard() {
   if [[ -z "$1" ]]; then
     # install_knative_serving_standard was called with no arg.
     build_knative_from_source
-    if (( INSTALL_BETA )); then
-      INSTALL_RELEASE_YAML="${SERVING_BETA_YAML}"
-    else
-      INSTALL_RELEASE_YAML="${SERVING_ALPHA_YAML}"
+    INSTALL_RELEASE_YAML="${SERVING_YAML}"
+
+    # install serving core if installing for Gloo or Kourier
+    if [[ -n "${GLOO_VERSION}" || -n "${KOURIER_VERSION}" ]]; then
+      INSTALL_RELEASE_YAML="${SERVING_CORE_YAML}"
     fi
+
     if (( INSTALL_MONITORING )); then
       INSTALL_MONITORING_YAML="${MONITORING_YAML}"
     fi
@@ -196,13 +219,21 @@ function install_knative_serving_standard() {
   echo "Cert Manager YAML: ${INSTALL_CERT_MANAGER_YAML}"
   echo "Knative YAML: ${INSTALL_RELEASE_YAML}"
 
-  if [[ -z "${GLOO_VERSION}" ]]; then
-    # install istio as the default knative ingress
+  # If no gateway was set on command line, assume Istio
+  if (( ! GATEWAY_SETUP )); then
+    echo ">> No gateway set up on command line, using Istio"
+    readonly ISTIO_VERSION="1.4-latest"
+  fi
+
+  if [[ -n "${ISTIO_VERSION}" ]]; then
     install_istio
-  else
-    # install gloo if $GLOO_VERSION is provided
+  fi
+  if [[ -n "${GLOO_VERSION}" ]]; then
     install_gloo
   fi
+  if [[ -n "${KOURIER_VERSION}" ]]; then
+    install_kourier
+  fi
 
   echo ">> Installing Cert-Manager"
   kubectl apply -f "${INSTALL_CERT_MANAGER_YAML}" --validate=false || return 1
@@ -210,13 +241,55 @@ function install_knative_serving_standard() {
   echo ">> Bringing up Serving"
   kubectl apply -f "${INSTALL_RELEASE_YAML}" || return 1
 
-  echo ">> Adding more activator pods."
-  # This command would fail if the HPA already exist, like during upgrade test.
-  # Therefore we don't exit on failure, and don't log an error message.
-  kubectl autoscale deploy --min=2 --max=2 -n knative-serving activator 2>/dev/null
+  if [[ -n "${KOURIER_VERSION}" ]]; then
+    echo ">> Making Kourier the default ingress"
+    cat <<EOF | kubectl apply -f -
+apiVersion: v1
+kind: ConfigMap
+metadata:
+  name: config-network
+  namespace: knative-serving
+  labels:
+    serving.knative.dev/release: devel
+data:
+  ingress.class: "kourier.ingress.networking.knative.dev"
+  clusteringress.class: "kourier.ingress.networking.knative.dev"
+EOF
+  fi
+
+  if (( RECONCILE_GATEWAY )); then
+    echo ">> Turning on reconcileExternalGateway"
+    cat <<EOF | kubectl apply -f -
+apiVersion: v1
+kind: ConfigMap
+metadata:
+  name: config-istio
+  namespace: knative-serving
+  labels:
+    serving.knative.dev/release: devel
+    networking.knative.dev/ingress-provider: istio
+data:
+  reconcileExternalGateway: "true"
+EOF
+  fi
+
+  echo ">> Turning on profiling.enable"
+  cat <<EOF | kubectl apply -f -
+apiVersion: v1
+kind: ConfigMap
+metadata:
+  name: config-observability
+  namespace: knative-serving
+data:
+  profiling.enable: "true"
+EOF
+
+  echo ">> Patching activator hpa"
+  # We set min replicas to 2 for testing multiple activator pods.
+  kubectl -n knative-serving patch hpa activator --patch '{"spec":{"minReplicas":2}}' || return 1
 
   # post-install steps for istio
-  if [[ -z "${GLOO_VERSION}" ]]; then
+  if [[ -n "${ISTIO_VERSION}" ]]; then
     # Due to the lack of Status in Istio, we have to ignore failures in initial requests.
     #
     # However, since network configurations may reach different ingress pods at slightly
@@ -233,16 +306,19 @@ function install_knative_serving_standard() {
     # There are reports of Envoy failing (503) when istio-pilot is overloaded.
     # We generously add more pilot instances here to verify if we can reduce flakes.
     if kubectl get hpa -n istio-system istio-pilot 2>/dev/null; then
-      # If HPA exists, update it.  Since patching will return non-zero if no change
-      # is made, we don't return on failure here.
       kubectl patch hpa -n istio-system istio-pilot \
-        --patch '{"spec": {"minReplicas": 3, "maxReplicas": 10, "targetCPUUtilizationPercentage": 60}}' \
-        `# Ignore error messages to avoid causing red herrings in the tests` \
-        2>/dev/null
+        --patch '{"spec": {"minReplicas": 3, "maxReplicas": 10, "targetCPUUtilizationPercentage": 60}}' || return 1
     else
       # Some versions of Istio don't provide an HPA for pilot.
       kubectl autoscale -n istio-system deploy istio-pilot --min=3 --max=10 --cpu-percent=60 || return 1
     fi
+  elif [[ -n "${GLOO_VERSION}" ]]; then
+    # Scale replicas of the Gloo proxies to handle large qps
+    kubectl scale -n gloo-system deployment knative-external-proxy --replicas=6
+    kubectl scale -n gloo-system deployment knative-internal-proxy --replicas=6
+  elif [[ -n "${KOURIER_VERSION}" ]]; then
+    # Scale replicas of the Kourier gateways to handle large qps
+    kubectl scale -n kourier-system deployment 3scale-kourier-gateway --replicas=6
   fi
 
   if [[ -n "${INSTALL_MONITORING_YAML}" ]]; then
@@ -259,6 +335,14 @@ function use_resolvable_domain() {
   echo "false"
 }
 
+# Check if we should use --https.
+function use_https() {
+  if (( HTTPS )); then
+    echo "--https"
+  else
+    echo ""
+  fi
+}
 
 # Uninstalls Knative Serving from the current cluster.
 function knative_teardown() {
@@ -294,8 +378,10 @@ function knative_teardown() {
 function test_setup() {
   echo ">> Setting up logging..."
 
-  # Install kail.
-  bash <( curl -sfL https://raw.githubusercontent.com/boz/kail/master/godownloader.sh) -b "$GOPATH/bin"
+  # Install kail if needed.
+  if ! which kail > /dev/null; then
+    bash <( curl -sfL https://raw.githubusercontent.com/boz/kail/master/godownloader.sh) -b "$GOPATH/bin"
+  fi
 
   # Capture all logs.
   kail > ${ARTIFACTS}/k8s.log.txt &
@@ -305,18 +391,31 @@ function test_setup() {
 
   echo ">> Creating test resources (test/config/)"
   ko apply ${KO_FLAGS} -f test/config/ || return 1
+  if (( MESH )); then
+    kubectl label namespace serving-tests istio-injection=enabled
+    kubectl label namespace serving-tests-alt istio-injection=enabled
+  fi
   ${REPO_ROOT_DIR}/test/upload-test-images.sh || return 1
   wait_until_pods_running knative-serving || return 1
-  if [[ -z "${GLOO_VERSION}" ]]; then
+  if [[ -n "${ISTIO_VERSION}" ]]; then
     wait_until_pods_running istio-system || return 1
     wait_until_service_has_external_ip istio-system istio-ingressgateway
-  else
+  fi
+  if [[ -n "${GLOO_VERSION}" ]]; then
     # we must set these override values to allow the test spoofing client to work with Gloo
     # see https://github.com/knative/pkg/blob/release-0.7/test/ingress/ingress.go#L37
-    export GATEWAY_OVERRIDE=clusteringress-proxy
+    export GATEWAY_OVERRIDE=knative-external-proxy
     export GATEWAY_NAMESPACE_OVERRIDE=gloo-system
     wait_until_pods_running gloo-system || return 1
-    wait_until_service_has_external_ip gloo-system clusteringress-proxy
+    wait_until_service_has_external_ip gloo-system knative-external-proxy
+  fi
+  if [[ -n "${KOURIER_VERSION}" ]]; then
+    # we must set these override values to allow the test spoofing client to work with Kourier
+    # see https://github.com/knative/pkg/blob/release-0.7/test/ingress/ingress.go#L37
+    export GATEWAY_OVERRIDE=kourier-external
+    export GATEWAY_NAMESPACE_OVERRIDE=kourier-system
+    wait_until_pods_running kourier-system || return 1
+    wait_until_service_has_external_ip kourier-system kourier-external
   fi
   if [[ -n "${INSTALL_MONITORING_YAML}" ]]; then
     wait_until_pods_running knative-monitoring || return 1
@@ -327,6 +426,7 @@ function test_setup() {
 function test_teardown() {
   echo ">> Removing test resources (test/config/)"
   ko delete --ignore-not-found=true --now -f test/config/
+  (( MESH )) && ko delete --ignore-not-found=true --now -f test/config/mtls/
   echo ">> Ensuring test namespaces are clean"
   kubectl delete all --all --ignore-not-found --now --timeout 60s -n serving-tests
   kubectl delete --ignore-not-found --now --timeout 60s namespace serving-tests
